# Generated by Django 5.1.4 on 2025-01-23 08:22

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("birder", "0002_alter_datahistory_options_alter_environment_options_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="monitor",
            name="verbosity",
        ),
    ]
