# Generated by Django 5.1.4 on 2025-01-22 19:39

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("birder", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="datahistory",
            options={"ordering": ["-date"]},
        ),
        migrations.AlterModelOptions(
            name="environment",
            options={"ordering": ["name"]},
        ),
        migrations.AlterModelOptions(
            name="project",
            options={"ordering": ["name"]},
        ),
        migrations.RemoveField(
            model_name="monitor",
            name="env",
        ),
        migrations.RemoveField(
            model_name="project",
            name="default",
        ),
        migrations.AddField(
            model_name="monitor",
            name="environment",
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to="birder.environment"),
        ),
        migrations.AddField(
            model_name="project",
            name="environments",
            field=models.ManyToManyField(related_name="projects", to="birder.environment"),
        ),
        migrations.AddField(
            model_name="project",
            name="icon",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
    ]
